% Example 5: Pattern Separation via Ellipses, p. 380f

addpath /Applications/SeDuMi_1_1
clc; clear all; close all;

% Data Set 1: cf. picture on p. 335
%P1 = [6.1 2.8; 5.9 2.92; 5.78 2.99; 6.04 2.98; 5.91 3.07; 6.07 3.08]'
%P2 = [5.5 2.87; 5.58 3; 5.48 3.13; 5.65 3.12; 5.7 3.22; 5.96 3.24; ...
%      6.17 3.22; 6.3 3.07]'
% Data Set 2: parallel strip
%P1 = [-2 1; -2 3; 0 3; 0 5; 2 5; 2 7]'
%P2 = [-2 0; -2 4; 0 2; 0 6; 2 4; 2 8]'
% Data Set 3: random points (cf. [Bo/Va], p. 429 ff)
n = 2; N = 100;
P1 = randn(n,N); P1 = P1*diag((0.95*rand(1,N))./sqrt(sum(P1.^2)));
P2 = randn(n,N); P2 = P2*diag((1.05+rand(1,N))./sqrt(sum(P2.^2)));
T = [1 -1; 2 1]; P1 = T*P1; P2 = T*P2;

% Solution via SeDuMi
[n,m1] = size(P1); m2 = size(P2,2); m = m1+m2;
P  = [P1 P2]; At = [];
for p = P; At = [At; vec([p; 1]*[p; 1]')']; end
A = [eye(m1)       zeros(m1,m2)  zeros(m1,1);
     zeros(m2,m1) -eye(m2)      -ones(m2,1)  ];
At = [A At];
b  = [ones(m1,1); zeros(m2,1)];
C  = zeros(n+1,n+1); c = [zeros(m,1); -1]; ct = [c; vec(C)];
K  = []; K.l = size(A,2); K.s = size(C,2);
[xs,ys,info] = sedumi(At,b,ct,K);
rho = xs(K.l)
Z = mat(xs(K.l+1:end),K.s)
X = Z(1:n,1:n)
xc =  - X \ Z(1:n,n+1) % center of 'ellipsoid'

% Displaying results
if n > 2; break; end;
F = @(x,y) X(1,1)*(x-xc(1)).^2 + 2*X(1,2)*(x-xc(1)).*(y-xc(2)) + ...
           X(2,2)*(y-xc(2)).^2; 
XX = linspace(min(P2(1,:))-0.1,max(P2(1,:))+0.1,50);
YY = linspace(min(P2(2,:))-0.25,max(P2(2,:))+0.05,50);
[X1,Y1] = ndgrid(XX,YY); F1 = F(X1,Y1);
FF1 = contour(X1,Y1,F1,[1 1],'b-'); hold on;
FF2 = contour(X1,Y1,F1,[rho rho],'k-'); hold on;
plot(P1(1,:),P1(2,:),'b+',P2(1,:),P2(2,:),'k*');
title('Pattern Separation via Ellipses');

