% Chapter3: Examples 1, 4, 6, 8, and 9

format long
global X y lambda;
[X,y] = LoadData;
[m,n] = size(X);


lambda = 1;

disp(' ');disp(' '); disp('lambda'); disp(lambda); disp(' '); disp(' ');


disp(' '); disp(' '); disp('NELDER-MEAD METHOD (Example 1)'); disp(' ');
options = optimset('Display','iter','TolX',1e-7,'TolFun',1e-7);
w0 = zeros(n,1);
[w,fval,exitflag,output] = fminsearch(@ObjFun,w0,options)
Table = [w0', ObjFun(w0)]; List = [15,30,45,60,75,90,110,135,168];
for k = List
  options = optimset('Display','off','TolX',1e-7,'TolFun',1e-7,'MaxIter',k);
  [w,fval,exitflag,output] = fminsearch(@ObjFun,w0,options);
  Table = [Table; w', fval];
end
Table



% Displaying results
F = @(u,v) w(1)*u+w(2)*v+w(3);
P1 = X(y==1,1:2);
P2 = X(y==-1,1:2);
XX = linspace(min(X(:,1))-0.5,max(X(:,1))+0.5,50);
YY = linspace(min(X(:,2))-0.5,max(X(:,2))+0.5,50);
[X1,Y1] = ndgrid(XX,YY); F1 = F(X1,Y1);
FF = contour(X1,Y1,F1,[0 0],'k-'); hold on;
plot(P1(:,1),P1(:,2),'bo',P2(:,1),P2(:,2),'k*');
title('Pattern Separation via Logistic Regression');


disp(' '); disp(' '); disp('STEEPEST DESCENT METHOD (Example 4)'); disp(' ');
options = optimset('Display','iter','TolX',1e-7,'TolFun',1e-7,'GradObj','on','LargeScale','off', ...
                   'HessUpdate','steepdesc','OutputFcn',@outfun);
w0 = zeros(n,1);
[w,fval,exitflag,output] = fminunc(@ObjFun,w0,options)
iterations = [iterations(2:end,:); w']
Table =[]; List = [0,2,4,6,14,24,34,44,54,66]+1;
for i = List
  w = iterations(i,:)'; [f,g] = ObjFun(w);
  Table = [Table; w', f, norm(g,inf)];
end
Table 



disp(' '); disp(' '); disp('(SUBSPACE) TRUST REGION METHOD (Example 6)'); disp(' ');
options = optimset('Display','iter','TolX',1e-7,'GradObj','on','Hessian','on','LargeScale','on', ...
                   'TolFun',1e-7,'OutputFcn',@outfun);
w0 = zeros(n,1); 
[w,fval,exitflag,output] = fminunc(@ObjFun,w0,options)
iterations
dim1 = size(iterations,1);
Table =[];
for i = 1:dim1
    w = iterations(i,:)'; [f,g] = ObjFun(w);
    Table = [Table; w', f, norm(g,inf)];
end
Table



disp(' '); disp(' '); disp('NEWTON (Example 8)'); disp(' ');
w0 = zeros(n,1); Tol = 1e-7;
[w,iterations] = LogReg(w0,Tol)
dim1 = size(iterations,1);
Table =[];
for i = 1:dim1
    w = iterations(i,:)'; [f,g] = ObjFun(w);  
    Table = [Table; w', f, norm(g,inf)];
end
Table



disp(' '); disp(' '); disp('DFP METHOD (Example 9)'); disp(' ');
options = optimset('Display','iter','TolX',1e-7,'GradObj','on','LargeScale','off',...
                   'HessUpdate','dfp','TolFun',1e-7,'OutputFcn',@outfun);
w0 = zeros(n,1); 
[w,fval,exitflag,output] = fminunc(@ObjFun,w0,options)
iterations = [iterations(2:end,:); w']
List = [0,2,4,6,14,23,33,43,52,62]+1;
Table =[];
for i = List
    w = iterations(i,:)'; [f,g] = ObjFun(w);
    Table = [Table; w', f, norm(g,inf)];
end
Table

    

disp(' '); disp(' '); disp('BFGS METHOD (Example 9)'); disp(' ');
options = optimset('Display','iter','TolX',1e-7,'GradObj','on','LargeScale','off',...
                   'HessUpdate','bfgs','TolFun',1e-7,'OutputFcn',@outfun);
w0 = zeros(n,1); 
[w,fval,exitflag,output] = fminunc(@ObjFun,w0,options)
iterations = [iterations(2:end,:); w']
List = [0,1,2,3,4,6,8,10,12,15]+1;
Table =[];
for i = List
    w = iterations(i,:)'; [f,g] = ObjFun(w);
    Table = [Table; w', f, norm(g,inf)];
end
Table


