function [x,u,J] = goldfarb(C,c,A,b)

% Experimental version of the Goldfarb-Idnani method
% for quadratic optimization
%
% Problem: min 1/2*x'*C*x + c'*x  s.t.  A'*x <= b

n = size(C,1);
x = C \ (-c);       % nabla(f) = 0  <==> C*x = -c
u = [];             % 0:0 - matrix 
lambda = 0; J = [];
[alpha_, p] = max(A'*x - b);

E = 10^(-13);
while (alpha_ > E*norm(A(:,p)))
  ap = A(:,p); AJ = A(:,J); J_  = [J, p];

  if (rank(AJ) < rank([AJ, ap]))  % Case 1
    AJ_ = A(:,J_); Z = zeros(length(J_));
    M = [ C, AJ_; AJ_', Z];
    rhs = [zeros(n,1); zeros(length(J),1); -1];
    xul = M \ rhs; x_ = xul(1:n);
    if (length(J) > 0) 
      u_ = xul(n+1:end-1); else u_ = [];
    end;
    lambda_ = xul(end:end);

    I = find(u_ < 0);
    if (length(I) > 0)
      [alpha_hat, i] = min(-u(I) ./ u_(I));
    end;

    if (length(I) == 0) | (alpha_ <= alpha_hat)  % Case 1a
      x = x + alpha_*x_; u = u + alpha_*u_;
      lambda = lambda + alpha_*lambda_;
      u = [u; lambda];
      lambda = 0; J = J_;
      [alpha_, p] = max(A'*x - b);

    else % Case 1b
      x = x + alpha_hat*x_; u = u + alpha_hat*u_;
      lambda = lambda + alpha_hat*lambda_;
      I = setdiff([1:length(J)],i);
      u = u(I); J = setdiff(J,J(i));
      alpha_ = alpha_ - alpha_hat;
    end;

  else  % Case 2
    v = AJ \ ap; I = find(v > 0);
    if (length(I) == 0)  % Case 2a
      error('There exists no feasible solution.')
    else  % Case 2b
      [lambda, i] = min(u(I) ./ v(I));
      I = setdiff([1:length(J)],i); 
      u = u(I) - lambda*v(I); J = setdiff(J,J(i));
    end;

  end;  % end if
end;  % end while

